package com.docus.log.aspect;

import com.docus.log.annotation.LogTrackGroup;
import com.docus.log.annotation.TrackGroup;
import com.docus.log.context.TrackContext;
import com.docus.log.context.TrackHelper;
import com.docus.log.processor.ITrackProcessor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;

import java.util.HashMap;

/**
 * @author linruifeng
 */
@Aspect
@Slf4j
public class TrackGroupAspect {

    @Autowired
    private ApplicationContext applicationContext;

    /**
     * 通用业务处理
     */
    @Around("@annotation(trackGroup)")
    public Object execute(final ProceedingJoinPoint joinPoint, TrackGroup trackGroup) throws Throwable {
        log.info("=== AOP @TrackGroup 切面启动器监听处理事件开始 ===");
        TrackContext context = getContext(joinPoint, trackGroup);

        ITrackProcessor processor = applicationContext.getAutowireCapableBeanFactory().createBean(trackGroup.processor());
        try {
            Object beforeResult = processor.beforeProcess(context);
            context.setBeforeResult(beforeResult);
            TrackContext.init(context.getParams());
            Object afterReturnResult = joinPoint.proceed();
            context.setAfterReturnResult(afterReturnResult);
            TrackHelper.setParams(context.getParams());
            return processor.process(context);
        } catch (Exception ex) {
            context.setError(true);
            context.setExMessageResult(ex.getMessage());
            return processor.process(context);
        } finally {
            processor.afterProcess(context);
            TrackContext.clear();
            log.info("=== AOP @TrackGroup 切面启动器监听处理事件结束 ===");
        }
    }

    /**
     * 通用日志处理
     */
    @Around("@annotation(logTrackGroup)")
    public Object execute(final ProceedingJoinPoint joinPoint, LogTrackGroup logTrackGroup) throws Throwable {
        log.info("=== AOP @LogTrackGroup 切面启动器监听处理事件开始 ===");
        final Object result;
        TrackContext context = getContext(joinPoint, logTrackGroup);
        ITrackProcessor processor = applicationContext.getAutowireCapableBeanFactory().createBean(logTrackGroup.processor());
        try {
            result = joinPoint.proceed();
            context.setAfterReturnResult(result);
            processor.process(context);
        } catch (Exception ex) {
            context.setError(true);
            context.setExMessageResult(ex.getMessage());
            processor.process(context);
            throw new RuntimeException(ex.getMessage());
        }
        log.info("=== AOP @LogTrackGroup 切面启动器监听处理事件结束 ===");
        return result;
    }

    private TrackContext getContext(final ProceedingJoinPoint joinPoint, TrackGroup logTrackGroup) {
        TrackContext context = new TrackContext();
        Signature signature = joinPoint.getSignature();
        context.setClassType(signature.getDeclaringType());
        context.setClassName(signature.getDeclaringTypeName());
        context.setMethodName(signature.getName());
        context.setArgs(joinPoint.getArgs());

        context.setGroup(logTrackGroup.group());
        context.setDesc(logTrackGroup.desc());
        context.setAction(logTrackGroup.action());
        context.setBeanNames(logTrackGroup.beanNames());
        context.setProcessor(logTrackGroup.processor());
        context.setParams(new HashMap<>());
        return context;
    }

    private TrackContext getContext(final ProceedingJoinPoint joinPoint, LogTrackGroup logTrackGroup) {
        TrackContext context = new TrackContext();
        Signature signature = joinPoint.getSignature();
        context.setClassType(signature.getDeclaringType());
        context.setClassName(signature.getDeclaringTypeName());
        context.setMethodName(signature.getName());
        context.setArgs(joinPoint.getArgs());

        context.setGroup(logTrackGroup.group());
        context.setDesc(logTrackGroup.desc());
        context.setAction(logTrackGroup.action());
        context.setProcessor(logTrackGroup.processor());
        context.setParams(new HashMap<>());
        return context;
    }

}
